"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntityManager = void 0;
var Entity_1 = require("./Entity");
var Query_1 = require("./Query");
var EntityManager = /** @class */ (function () {
    function EntityManager() {
        var _this = this;
        this.changed = [];
        this.removed = [];
        this.queries = {
            '': new Query_1.Query([], [])
        };
        this.scheduleUpdate = function (entity) { return _this.changed.push(entity); };
    }
    EntityManager.prototype.create = function () {
        return new Entity_1.Entity(this.scheduleUpdate);
    };
    EntityManager.prototype.query = function () {
        var components = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            components[_i] = arguments[_i];
        }
        var queryId = getQueryId(components);
        if (!this.queries[queryId]) {
            this.queries[queryId] = new Query_1.Query(components, this.queries[''].entities);
        }
        return this.queries[queryId].entities;
    };
    EntityManager.prototype.queryOne = function () {
        var components = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            components[_i] = arguments[_i];
        }
        return this.query.apply(this, components)[0];
    };
    EntityManager.prototype.scheduleRemove = function (entity) {
        this.removed.push(entity);
    };
    EntityManager.prototype.processUpdates = function () {
        var _loop_1 = function (query) {
            this_1.changed.forEach(function (entity) { return query.onChange(entity); });
            this_1.removed.forEach(function (entity) { return query.onRemove(entity); });
        };
        var this_1 = this;
        for (var _i = 0, _a = Object.values(this.queries); _i < _a.length; _i++) {
            var query = _a[_i];
            _loop_1(query);
        }
        this.changed.forEach(function (entity) { entity['didNotify'] = false; });
        this.changed.length = 0;
        this.removed.forEach(function (entity) { entity['didNotify'] = true; });
        this.removed.length = 0;
    };
    return EntityManager;
}());
exports.EntityManager = EntityManager;
function getQueryId(components) {
    return components.sort().join('+');
}
//# sourceMappingURL=EntityManager.js.map