import { EntityManager } from './EntityManager';
import { PriorityQueue } from './PriorityQueue';
var World = /** @class */ (function () {
    function World(systems) {
        this.systems = systems;
        this.entityManager = new EntityManager();
        this.running = false;
        this.events = new PriorityQueue();
    }
    World.prototype.query = function () {
        var _a;
        var components = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            components[_i] = arguments[_i];
        }
        this.ensureRunning();
        return (_a = this.entityManager).query.apply(_a, components);
    };
    World.prototype.queryOne = function () {
        var components = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            components[_i] = arguments[_i];
        }
        this.ensureRunning();
        return this.query.apply(this, components)[0];
    };
    World.prototype.create = function () {
        this.ensureRunning();
        return this.entityManager.create();
    };
    World.prototype.remove = function (entity) {
        this.ensureRunning();
        return this.entityManager.scheduleRemove(entity);
    };
    World.prototype.emit = function (event, priority) {
        if (priority === void 0) { priority = 0; }
        this.ensureRunning();
        this.events.enqueue(event, priority);
    };
    World.prototype.run = function (callback) {
        if (this.running) {
            callback();
            return;
        }
        this.running = true;
        callback();
        this.entityManager.processUpdates();
        while (!this.events.isEmpty()) {
            var event_1 = this.events.dequeue();
            for (var _i = 0, _a = this.systems; _i < _a.length; _i++) {
                var system = _a[_i];
                system(this, event_1);
                this.entityManager.processUpdates();
            }
        }
        this.running = false;
    };
    World.prototype.ensureRunning = function () {
        if (!this.running) {
            throw new Error('Outside synchronous systems you need to wrap calls to world in world.run()');
        }
    };
    return World;
}());
export { World };
//# sourceMappingURL=World.js.map