import { system } from './System';
import { InitEvent, UpdateTick, RenderTick } from './events';
function scheduleRaf(callback) {
    var id = requestAnimationFrame(callback);
    return function () { return cancelAnimationFrame(id); };
}
function scheduleTimeout(callback) {
    var id = setTimeout(callback, 16);
    return function () { return clearTimeout(id); };
}
var scheduleDefault = typeof requestAnimationFrame === 'function' ? scheduleRaf : scheduleTimeout;
export function gameClock(ticksPerSecond, maxDroppedTicks, schedule) {
    if (ticksPerSecond === void 0) { ticksPerSecond = 60; }
    if (maxDroppedTicks === void 0) { maxDroppedTicks = ticksPerSecond; }
    if (schedule === void 0) { schedule = scheduleDefault; }
    var deltaMs = 1000 / ticksPerSecond;
    return system(InitEvent, function (world) {
        var lastTime = Date.now();
        var totalUpdates = 0;
        update();
        function update() {
            world.run(function () {
                var now = Date.now();
                var droppedTicks = 0;
                while (lastTime <= now) {
                    droppedTicks++;
                    if (droppedTicks <= maxDroppedTicks) {
                        totalUpdates++;
                        world.emit(new UpdateTick(now, totalUpdates / ticksPerSecond, 1 / ticksPerSecond));
                    }
                    lastTime += deltaMs;
                }
                world.emit(new RenderTick());
            });
            schedule(update);
        }
    });
}
//# sourceMappingURL=gameClock.js.map